@@ -62,8 +62,11 @@ int callback (const struct _u_request * request, struct _u_response * response,
   char request_body[request->binary_body_length + 1];
   o_strncpy(request_body, request->binary_body, request->binary_body_length);
   request_body[request->binary_body_length] = '\0';
-  response->binary_body = o_strdup("ok");
-  response->binary_body_length = o_strlen("ok");
+  if (o_strstr(request->http_url, "limit") != NULL) {
+    ulfius_set_string_body_response(response, 200, "This is a large body response that should be truncated by the client");
+  } else {
+    ulfius_set_string_body_response(response, 200, "ok");
+  }
   response->status = 200;
   printf("######################################################\n###################### Callback ######################\n######################################################\n\nMethod is %s\n  url is %s\n\n  parameters from the url are \n%s\n\n  cookies are \n%s\n\n  headers are \n%s\n\n  post parameters are \n%s\n\n  raw body is \n%s\n\n  user data is %s\n\n",
                                   request->http_verb, request->http_url, url_params, cookies, headers, post_params, request_body, (char *)user_data);
