@@ -7,26 +7,29 @@
  * As a result, the index.html page will increment a sheep counter and 
  * shows a new sheep on the screen every time an increment is done (every 5 seconds)
  * 
- * Copyright 2015-2017 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2015-2022 Nicolas Mora <mail@babelouest.org>
  * 
  * License MIT
  *
  */
 
-#include <string.h>
 #include <jansson.h>
-
+#include <string.h>
 #include <ulfius.h>
-#include <u_example.h>
 
-#include "static_compressed_inmemory_website_callback.h"
 #include "http_compression_callback.h"
+#include "static_compressed_inmemory_website_callback.h"
+#include "u_example.h"
 
 #define PORT 7437
 #define PREFIX "/sheep"
 #define FILE_PREFIX "/upload"
 #define STATIC_FOLDER "static"
 
+#if defined(U_DISABLE_JANSSON)
+#error You must build ulfius with jansson support enabled to compile this example, check the install documentation
+#else
+
 /**
  * decode a u_map into a string
  */
@@ -38,17 +41,17 @@ char * print_map(const struct _u_map * map) {
     keys = u_map_enum_keys(map);
     for (i=0; keys[i] != NULL; i++) {
       value = u_map_get(map, keys[i]);
-      len = snprintf(NULL, 0, "key is %s, length is %zu, value is %.*s", keys[i], u_map_get_length(map, keys[i]), (int)u_map_get_length(map, keys[i]), value);
-      line = o_malloc((len+1)*sizeof(char));
-      snprintf(line, (len+1), "key is %s, length is %zu, value is %.*s", keys[i], u_map_get_length(map, keys[i]), (int)u_map_get_length(map, keys[i]), value);
+      len = snprintf(NULL, 0, "key is %s, value is %s", keys[i], value);
+      line = o_malloc((size_t)(len+1));
+      snprintf(line, (size_t)(len+1), "key is %s, value is %s", keys[i], value);
       if (to_return != NULL) {
-        len = o_strlen(to_return) + o_strlen(line) + 1;
-        to_return = o_realloc(to_return, (len+1)*sizeof(char));
+        len = (int)(o_strlen(to_return) + o_strlen(line) + 1);
+        to_return = o_realloc(to_return, (size_t)(len+1));
         if (o_strlen(to_return) > 0) {
           strcat(to_return, "\n");
         }
       } else {
-        to_return = o_malloc((o_strlen(line) + 1)*sizeof(char));
+        to_return = o_malloc((o_strlen(line) + 1));
         to_return[0] = 0;
       }
       strcat(to_return, line);
@@ -91,6 +94,7 @@ static int callback_sheep_counter_reset (const struct _u_request * request, stru
   json_t * json_body = NULL;
   json_int_t * nb_sheep = user_data;
   * nb_sheep = 0;
+  (void)(request);
   
   json_body = json_object();
   json_object_set_new(json_body, "nbsheep", json_integer(0));
@@ -107,6 +111,7 @@ static int callback_sheep_counter_reset (const struct _u_request * request, stru
 static int callback_sheep_counter_add (const struct _u_request * request, struct _u_response * response, void * user_data) {
   json_t * json_body = NULL;
   json_int_t * nb_sheep = user_data;
+  (void)(request);
   
   (*nb_sheep)++;
   
@@ -127,6 +132,9 @@ static int callback_upload_file (const struct _u_request * request, struct _u_re
 
   char * string_body = msprintf("Upload file\n\n  method is %s\n  url is %s\n\n  parameters from the url are \n%s\n\n  cookies are \n%s\n\n  headers are \n%s\n\n  post parameters are \n%s\n\n",
                                   request->http_verb, request->http_url, url_params, cookies, headers, post_params);
+  (void)(response);
+  (void)(user_data);
+
   y_log_message(Y_LOG_LEVEL_DEBUG, "Post parameters:\n%s", post_params);
   y_log_message(Y_LOG_LEVEL_DEBUG, "URL parameters:\n%s", url_params);
   o_free(url_params);
@@ -143,6 +151,7 @@ static int callback_upload_file (const struct _u_request * request, struct _u_re
 static int callback_form_submit (const struct _u_request * request, struct _u_response * response, void * user_data) {
   char * post_params = print_map(request->map_post_body),
        * url_params = print_map(request->map_url);
+  (void)(user_data);
   y_log_message(Y_LOG_LEVEL_DEBUG, "Post parameters:\n%s", post_params);
   y_log_message(Y_LOG_LEVEL_DEBUG, "URL parameters:\n%s", url_params);
   o_free(post_params);
@@ -163,6 +172,10 @@ static int file_upload_callback (const struct _u_request * request,
                           uint64_t off, 
                           size_t size, 
                           void * cls) {
+  (void)(request);
+  (void)(content_type);
+  (void)(transfer_encoding);
+  (void)(data);
   y_log_message(Y_LOG_LEVEL_DEBUG, "Got from file '%s' of the key '%s', offset %llu, size %zu, cls is '%s'", filename, key, off, size, cls);
   return U_OK;
 }
@@ -170,7 +183,7 @@ static int file_upload_callback (const struct _u_request * request,
 /**
  * Main function
  */
-int main (int argc, char **argv) {
+int main (void) {
   
   struct _u_compressed_inmemory_website_config file_config;
   json_int_t nb_sheep = 0;
@@ -193,7 +206,7 @@ int main (int argc, char **argv) {
     u_map_put(&file_config.mime_types, ".map", "application/octet-stream");
     u_map_put(&file_config.mime_types, ".json", "application/json");
     u_map_put(&file_config.mime_types, "*", "application/octet-stream");
-    file_config.files_path = "static";
+    file_config.files_path = realpath("static", NULL);
     file_config.url_prefix = FILE_PREFIX;
 
     if (ulfius_init_instance(&instance, PORT, NULL, NULL) != U_OK) {
@@ -233,9 +246,11 @@ int main (int argc, char **argv) {
     ulfius_stop_framework(&instance);
     ulfius_clean_instance(&instance);
     u_clean_compressed_inmemory_website_config(&file_config);
+    free(file_config.files_path);
   }
   
   y_close_logs();
   
   return 0;
 }
+#endif
