@@ -5,22 +5,23 @@
  * This example program describes the main features 
  * that are available in a callback function
  * 
- * Copyright 2015-2017 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2015-2022 Nicolas Mora <mail@babelouest.org>
  * 
  * License MIT
  *
  */
 
-#include <string.h>
-
 #ifndef _WIN32
   #include <sys/socket.h>
-  #include <netinet/in.h>
+
   #include <arpa/inet.h>
+  #include <netinet/in.h>
 #endif
 
+#include <string.h>
 #include <ulfius.h>
-#include <u_example.h>
+
+#include "u_example.h"
 
 #define PORT 8537
 #define PREFIX "/test"
@@ -54,16 +55,16 @@ char * print_map(const struct _u_map * map) {
     for (i=0; keys[i] != NULL; i++) {
       value = u_map_get(map, keys[i]);
       len = snprintf(NULL, 0, "key is %s, value is %s", keys[i], value);
-      line = o_malloc((len+1)*sizeof(char));
-      snprintf(line, (len+1), "key is %s, value is %s", keys[i], value);
+      line = o_malloc((size_t)(len+1));
+      snprintf(line, (size_t)(len+1), "key is %s, value is %s", keys[i], value);
       if (to_return != NULL) {
-        len = o_strlen(to_return) + o_strlen(line) + 1;
-        to_return = o_realloc(to_return, (len+1)*sizeof(char));
+        len = (int)(o_strlen(to_return) + o_strlen(line) + 1);
+        to_return = o_realloc(to_return, (size_t)(len+1));
         if (o_strlen(to_return) > 0) {
           strcat(to_return, "\n");
         }
       } else {
-        to_return = o_malloc((o_strlen(line) + 1)*sizeof(char));
+        to_return = o_malloc((o_strlen(line) + 1));
         to_return[0] = 0;
       }
       strcat(to_return, line);
@@ -75,22 +76,27 @@ char * print_map(const struct _u_map * map) {
   }
 }
 
-char * read_file(const char * filename) {
+static char * read_file(const char * filename) {
   char * buffer = NULL;
   long length;
-  FILE * f = fopen (filename, "rb");
-  if (f != NULL) {
-    fseek (f, 0, SEEK_END);
-    length = ftell (f);
-    fseek (f, 0, SEEK_SET);
-    buffer = o_malloc (length + 1);
-    if (buffer != NULL) {
-      fread (buffer, 1, length, f);
-      buffer[length] = '\0';
+  FILE * f;
+  if (filename != NULL) {
+    f = fopen (filename, "rb");
+    if (f) {
+      fseek (f, 0, SEEK_END);
+      length = ftell (f);
+      fseek (f, 0, SEEK_SET);
+      buffer = o_malloc ((size_t)(length + 1));
+      if (buffer != NULL) {
+        fread (buffer, 1, (size_t)length, f);
+        buffer[length] = '\0';
+      }
+      fclose (f);
     }
-    fclose (f);
+    return buffer;
+  } else {
+    return NULL;
   }
-  return buffer;
 }
 
 int main (int argc, char **argv) {
@@ -159,6 +165,8 @@ int main (int argc, char **argv) {
  * Callback function that put a "Hello World!" string in the response
  */
 int callback_get_test (const struct _u_request * request, struct _u_response * response, void * user_data) {
+  (void)(request);
+  (void)(user_data);
   ulfius_set_string_body_response(response, 200, "Hello World!");
   return U_CALLBACK_CONTINUE;
 }
@@ -167,6 +175,9 @@ int callback_get_test (const struct _u_request * request, struct _u_response * r
  * Callback function that put an empty response and a status 200
  */
 int callback_get_empty_response (const struct _u_request * request, struct _u_response * response, void * user_data) {
+  (void)(request);
+  (void)(user_data);
+  (void)(response);
   return U_CALLBACK_CONTINUE;
 }
 
@@ -176,6 +187,7 @@ int callback_get_empty_response (const struct _u_request * request, struct _u_re
 int callback_post_test (const struct _u_request * request, struct _u_response * response, void * user_data) {
   char * post_params = print_map(request->map_post_body);
   char * response_body = msprintf("Hello World!\n%s", post_params);
+  (void)(user_data);
   ulfius_set_string_body_response(response, 200, response_body);
   o_free(response_body);
   o_free(post_params);
@@ -209,6 +221,7 @@ int callback_all_test_foo (const struct _u_request * request, struct _u_response
  * The counter cookie is incremented every time the client reloads this url
  */
 int callback_get_cookietest (const struct _u_request * request, struct _u_response * response, void * user_data) {
+  (void)(user_data);
   const char * lang = u_map_get(request->map_url, "lang"), * extra = u_map_get(request->map_url, "extra"), 
              * counter = u_map_get(request->map_cookie, "counter");
   char new_counter[8];
@@ -217,7 +230,7 @@ int callback_get_cookietest (const struct _u_request * request, struct _u_respon
   if (counter == NULL) {
     i_counter = 0;
   } else {
-    i_counter = strtol(counter, NULL, 10);
+    i_counter = (int)strtol(counter, NULL, 10);
     i_counter++;
   }
   snprintf(new_counter, 7, "%d", i_counter);
@@ -233,6 +246,8 @@ int callback_get_cookietest (const struct _u_request * request, struct _u_respon
  * Default callback function called if no endpoint has a match
  */
 int callback_default (const struct _u_request * request, struct _u_response * response, void * user_data) {
+  (void)(request);
+  (void)(user_data);
   ulfius_set_string_body_response(response, 404, "Page not found, do what you want");
   return U_CALLBACK_CONTINUE;
 }
