@@ -6,7 +6,7 @@
  *
  * u_private.h: private structures and functions declarations
  *
- * Copyright 2015-2017 Nicolas Mora <mail@babelouest.org>
+ * Copyright 2015-2022 Nicolas Mora <mail@babelouest.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public License
@@ -26,8 +26,6 @@
 #ifndef __U_PRIVATE_H__
 #define __U_PRIVATE_H__
 
-#include "ulfius.h"
-
 /** Macro to avoid compiler warning when some parameters are unused and that's ok **/
 #define UNUSED(x) (void)(x)
 
@@ -45,6 +43,7 @@
   #endif // U_WITH_LWIP
 #endif // U_WITH_FREERTOS
 
+#include <ulfius.h>
 
 /**********************************
  * Internal functions declarations
@@ -147,6 +146,7 @@ int ulfius_check_list_match(const char * source, const char * match, const char
  */
 int ulfius_check_first_match(const char * source, const char * match, const char * separator, char ** result);
 
+#ifndef U_DISABLE_WS_MESSAGE_LIST
 /**
  * Initialize a websocket message list
  * Return U_OK on success
@@ -163,6 +163,7 @@ void ulfius_clear_websocket_message_list(struct _websocket_message_list * messag
  * Return U_OK on success
  */
 int ulfius_push_websocket_message(struct _websocket_message_list * message_list, struct _websocket_message * message);
+#endif
 
 /**
  * Clear data of a websocket
