@@ -1,14 +1,14 @@
 /* Public domain, no copyright. Use at your own risk. */
 
+#include <check.h>
+#include <errno.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <errno.h>
 #include <time.h>
-#include <zlib.h>
-
-#include <check.h>
 #include <ulfius.h>
+#include <zlib.h>
+#include <curl/curl.h>
 
 #define MIN(A, B) ((A)>(B)?(B):(A))
 
@@ -25,6 +25,8 @@
 #define PORT_7 9281
 #define PORT_8 9282
 #define PORT_9 9283
+#define PORT_10 9284
+#define PORT_11 9285
 #define PREFIX_WEBSOCKET "/websocket"
 #define MESSAGE "HelloFrom"
 #define MESSAGE_CLIENT "HelloFromClient"
@@ -33,11 +35,16 @@
 #define MESSAGE_EXT2 "Plop"
 #define MESSAGE_EXT3 "Gnaa"
 #define MESSAGE_EXT4 "Glop"
+#define ORIGIN "http://client.ulfius.tld"
+#define NO_ORIGIN "null"
 #define _U_W_BUFF_LEN 256
 
 #define U_W_FLAG_SERVER  0x00000001
 #define U_W_FLAG_CONTEXT 0x00000010
 
+static pthread_mutex_t ws_lock;
+static pthread_cond_t  ws_cond;
+
 #ifndef U_DISABLE_WEBSOCKET
 
 void websocket_manager_callback_empty (const struct _u_request * request, struct _websocket_manager * websocket_manager, void * websocket_manager_user_data) {
@@ -54,14 +61,24 @@ void websocket_onclose_callback_client (const struct _u_request * request, struc
 }
 
 void websocket_echo_message_callback (const struct _u_request * request,
-                                         struct _websocket_manager * websocket_manager,
-                                         const struct _websocket_message * last_message,
-                                         void * websocket_incoming_message_user_data) {
+                                       struct _websocket_manager * websocket_manager,
+                                       const struct _websocket_message * last_message,
+                                       void * websocket_incoming_message_user_data) {
   if (last_message->opcode == U_WEBSOCKET_OPCODE_TEXT) {
     ck_assert_int_eq(ulfius_websocket_send_fragmented_message(websocket_manager, U_WEBSOCKET_OPCODE_TEXT, last_message->data_len, last_message->data, last_message->fragment_len), U_OK);
   }
 }
 
+void websocket_origin_callback (const struct _u_request * request,
+                                   struct _websocket_manager * websocket_manager,
+                                   void * websocket_incoming_message_user_data) {
+  if (websocket_incoming_message_user_data != NULL) {
+    ulfius_websocket_send_message(websocket_manager, U_WEBSOCKET_OPCODE_TEXT, o_strlen((char *)websocket_incoming_message_user_data), (char *)websocket_incoming_message_user_data);
+  } else {
+    ulfius_websocket_send_message(websocket_manager, U_WEBSOCKET_OPCODE_TEXT, o_strlen(NO_ORIGIN), NO_ORIGIN);
+  }
+}
+
 void websocket_onclose_message_callback (const struct _u_request * request, struct _websocket_manager * websocket_manager, void * websocket_onclose_user_data) {
   o_free(websocket_onclose_user_data);
 }
@@ -94,6 +111,68 @@ void websocket_incoming_message_callback_client (const struct _u_request * reque
   ck_assert_int_eq(0, o_strncmp(message->data, DEFAULT_MESSAGE, message->data_len));
 }
 
+void websocket_incoming_json_valid_messages (const struct _u_request * request, struct _websocket_manager * websocket_manager, const struct _websocket_message * message, void * websocket_incoming_user_data) {
+  json_t * j_message = ulfius_websocket_parse_json_message(message, NULL), * j_expected_message = json_pack("{ss}", "message", "Hello World!");
+  
+  // Valid JSON
+  ck_assert_ptr_ne(j_message, NULL);
+  ck_assert_int_eq(1, json_equal(j_message, j_expected_message));
+  ck_assert_int_ne(message->data_len, 0);
+
+  json_decref(j_message);
+  json_decref(j_expected_message);
+}
+
+void websocket_incoming_json_invalid_messages (const struct _u_request * request, struct _websocket_manager * websocket_manager, const struct _websocket_message * message, void * websocket_incoming_user_data) {
+  json_t * j_message = ulfius_websocket_parse_json_message(message, NULL);
+  
+  // Invalid JSON
+  ck_assert_ptr_eq(j_message, NULL);
+  ck_assert_int_ne(message->data_len, 0);
+
+  json_decref(j_message);
+}
+
+void websocket_incoming_json_empty_messages (const struct _u_request * request, struct _websocket_manager * websocket_manager, const struct _websocket_message * message, void * websocket_incoming_user_data) {
+  json_t * j_message = ulfius_websocket_parse_json_message(message, NULL);
+  
+  // Empty payload
+  ck_assert_ptr_eq(j_message, NULL);
+
+  json_decref(j_message);
+}
+
+void websocket_manager_json_valid_callback(const struct _u_request * request, struct _websocket_manager * websocket_manager, void * websocket_manager_user_data) {
+  json_t * j_message = json_pack("{ss}", "message", "Hello World!");
+  ck_assert_ptr_ne(j_message, NULL);
+  const char str_message[] = "{\"message\":\"Hello World!\"}";
+  
+  ck_assert_int_eq(ulfius_websocket_send_json_message(websocket_manager, j_message), U_OK);
+  ck_assert_int_eq(ulfius_websocket_send_message(websocket_manager, U_WEBSOCKET_OPCODE_TEXT, o_strlen(str_message), str_message), U_OK);
+  ck_assert_int_eq(ulfius_websocket_send_message(websocket_manager, U_WEBSOCKET_OPCODE_BINARY, o_strlen(str_message), str_message), U_OK);
+
+  json_decref(j_message);
+}
+
+void websocket_manager_json_invalid_callback(const struct _u_request * request, struct _websocket_manager * websocket_manager, void * websocket_manager_user_data) {
+  ck_assert_int_eq(ulfius_websocket_send_message(websocket_manager, U_WEBSOCKET_OPCODE_TEXT, o_strlen(DEFAULT_MESSAGE), DEFAULT_MESSAGE), U_OK);
+  ck_assert_int_eq(ulfius_websocket_send_message(websocket_manager, U_WEBSOCKET_OPCODE_BINARY, o_strlen(DEFAULT_MESSAGE), DEFAULT_MESSAGE), U_OK);
+}
+
+void websocket_manager_json_empty_callback(const struct _u_request * request, struct _websocket_manager * websocket_manager, void * websocket_manager_user_data) {
+  ck_assert_int_eq(ulfius_websocket_send_json_message(websocket_manager, NULL), U_ERROR_PARAMS);
+  ck_assert_int_eq(ulfius_websocket_send_message(websocket_manager, U_WEBSOCKET_OPCODE_TEXT, 0, NULL), U_OK);
+  ck_assert_int_eq(ulfius_websocket_send_message(websocket_manager, U_WEBSOCKET_OPCODE_BINARY, 0, NULL), U_OK);
+}
+
+void websocket_incoming_message_without_origin_callback_client (const struct _u_request * request, struct _websocket_manager * websocket_manager, const struct _websocket_message * message, void * websocket_incoming_user_data) {
+  ck_assert_int_eq(0, o_strncmp(message->data, NO_ORIGIN, message->data_len));
+}
+
+void websocket_incoming_message_with_origin_callback_client (const struct _u_request * request, struct _websocket_manager * websocket_manager, const struct _websocket_message * message, void * websocket_incoming_user_data) {
+  ck_assert_int_eq(0, o_strncmp(message->data, ORIGIN, message->data_len));
+}
+
 void websocket_manager_extension_callback (const struct _u_request * request, struct _websocket_manager * websocket_manager, void * websocket_manager_user_data) {
   y_log_message(Y_LOG_LEVEL_DEBUG, "server message '%s' sent", MESSAGE_SERVER);
   ck_assert_int_eq(ulfius_websocket_send_message(websocket_manager, U_WEBSOCKET_OPCODE_TEXT, o_strlen(MESSAGE_SERVER), MESSAGE_SERVER), U_OK);
@@ -167,6 +246,7 @@ void websocket_incoming_extension_deflate_server_callback_disabled (const struct
   ck_assert_int_eq(last_message->rsv&U_WEBSOCKET_RSV1, 0);
 }
 
+#ifndef U_DISABLE_WS_MESSAGE_LIST
 void websocket_manager_callback_keep_messages (const struct _u_request * request, struct _websocket_manager * websocket_manager, void * websocket_manager_user_data) {
   if (websocket_manager_user_data != NULL) {
     websocket_manager->keep_messages = *((int*)(websocket_manager_user_data));
@@ -203,6 +283,11 @@ void websocket_onclose_callback_keep_messages (const struct _u_request * request
     ck_assert_int_eq(websocket_manager->message_list_outcoming->len, 0);
   }
 }
+#endif
+
+void websocket_origin_onclose_callback (const struct _u_request * request, struct _websocket_manager * websocket_manager, void * websocket_onclose_user_data) {
+  o_free(websocket_onclose_user_data);
+}
 
 int websocket_extension1_message_out_perform(const uint8_t opcode, const uint64_t data_len_in, const char * data_in, uint64_t * data_len_out, char ** data_out, const uint64_t fragment_len, void * user_data, void * context) {
   ck_assert_int_eq(opcode, U_WEBSOCKET_OPCODE_TEXT);
@@ -296,6 +381,46 @@ int callback_websocket (const struct _u_request * request, struct _u_response *
   return (ret == U_OK)?U_CALLBACK_CONTINUE:U_CALLBACK_ERROR;
 }
 
+int callback_websocket_with_origin (const struct _u_request * request, struct _u_response * response, void * user_data) {
+  int ret;
+  char * origin = o_strdup(u_map_get_case(request->map_header, "Origin"));
+
+  if (u_map_get_case(request->map_header, "Origin") != NULL) {
+    ulfius_set_string_body_response(response, 400, u_map_get_case(request->map_header, "Origin"));
+  } else {
+    response->status = 400;
+  }
+  ret = ulfius_set_websocket_response(response, NULL, NULL, &websocket_origin_callback, origin, NULL, NULL, &websocket_origin_onclose_callback, origin);
+  ck_assert_int_eq(ret, U_OK);
+
+  return (ret == U_OK)?U_CALLBACK_CONTINUE:U_CALLBACK_ERROR;
+}
+
+int callback_websocket_json_valid_messages (const struct _u_request * request, struct _u_response * response, void * user_data) {
+  int ret;
+  
+  ret = ulfius_set_websocket_response(response, NULL, NULL, NULL, NULL, websocket_incoming_json_valid_messages, user_data, NULL, NULL);
+  ck_assert_int_eq(ret, U_OK);
+  return (ret == U_OK)?U_CALLBACK_CONTINUE:U_CALLBACK_ERROR;
+}
+
+int callback_websocket_json_invalid_messages (const struct _u_request * request, struct _u_response * response, void * user_data) {
+  int ret;
+  
+  ret = ulfius_set_websocket_response(response, NULL, NULL, NULL, NULL, websocket_incoming_json_invalid_messages, user_data, NULL, NULL);
+  ck_assert_int_eq(ret, U_OK);
+  return (ret == U_OK)?U_CALLBACK_CONTINUE:U_CALLBACK_ERROR;
+}
+
+int callback_websocket_json_empty_messages (const struct _u_request * request, struct _u_response * response, void * user_data) {
+  int ret;
+  
+  ret = ulfius_set_websocket_response(response, NULL, NULL, NULL, NULL, websocket_incoming_json_empty_messages, user_data, NULL, NULL);
+  ck_assert_int_eq(ret, U_OK);
+  return (ret == U_OK)?U_CALLBACK_CONTINUE:U_CALLBACK_ERROR;
+}
+
+#ifndef U_DISABLE_WS_MESSAGE_LIST
 int callback_websocket_keep_messages (const struct _u_request * request, struct _u_response * response, void * user_data) {
   int ret;
   
@@ -303,6 +428,7 @@ int callback_websocket_keep_messages (const struct _u_request * request, struct
   ck_assert_int_eq(ret, U_OK);
   return (ret == U_OK)?U_CALLBACK_CONTINUE:U_CALLBACK_ERROR;
 }
+#endif
 
 int callback_websocket_onclose (const struct _u_request * request, struct _u_response * response, void * user_data) {
   int ret;
@@ -634,7 +760,8 @@ START_TEST(test_ulfius_websocket_client)
   ulfius_clean_request(&request);
   ulfius_clean_response(&response);
   
-  // Test incorrect websocket connection on correct websocket service
+#if LIBCURL_VERSION_NUM < 0x075600
+  // Test incorrect websocket connection on correct websocket service for libcurl < 7.86.0
   ulfius_init_request(&request);
   ulfius_init_response(&response);
   request.http_verb = o_strdup("GET");
@@ -642,6 +769,7 @@ START_TEST(test_ulfius_websocket_client)
   ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_ERROR_LIBCURL); // On a websocket connection attempt, libcurl return 'Unsupported protocol'
   ulfius_clean_request(&request);
   ulfius_clean_response(&response);
+#endif
   
   // Test incorrect websocket connection on correct websocket service
   ulfius_init_request(&request);
@@ -682,7 +810,8 @@ START_TEST(test_ulfius_websocket_client_no_onclose)
   ulfius_clean_request(&request);
   ulfius_clean_response(&response);
   
-  // Test incorrect websocket connection on correct websocket service
+#if LIBCURL_VERSION_NUM < 0x075600
+  // Test incorrect websocket connection on correct websocket service for libcurl < 7.86.0
   ulfius_init_request(&request);
   ulfius_init_response(&response);
   request.http_verb = o_strdup("GET");
@@ -690,6 +819,7 @@ START_TEST(test_ulfius_websocket_client_no_onclose)
   ck_assert_int_eq(ulfius_send_http_request(&request, &response), U_ERROR_LIBCURL); // On a websocket connection attempt, libcurl return 'Unsupported protocol'
   ulfius_clean_request(&request);
   ulfius_clean_response(&response);
+#endif
   
   // Test incorrect websocket connection on correct websocket service
   ulfius_init_request(&request);
@@ -863,6 +993,7 @@ START_TEST(test_ulfius_websocket_extension_deflate_error_params)
 }
 END_TEST
 
+#ifndef U_DISABLE_WS_MESSAGE_LIST
 START_TEST(test_ulfius_websocket_keep_messages)
 {
   struct _u_instance instance;
@@ -923,6 +1054,102 @@ START_TEST(test_ulfius_websocket_keep_messages)
   ulfius_clean_instance(&instance);
 }
 END_TEST
+#endif
+
+START_TEST(test_ulfius_websocket_origin)
+{
+  struct _u_instance instance;
+  struct _u_request request;
+  struct _u_response response;
+  struct _websocket_client_handler websocket_client_handler = {NULL, NULL};
+  char url[64];
+
+  ck_assert_int_eq(ulfius_init_instance(&instance, PORT_10, NULL, NULL), U_OK);
+  ck_assert_int_eq(ulfius_add_endpoint_by_val(&instance, "GET", PREFIX_WEBSOCKET, NULL, 0, &callback_websocket_with_origin, NULL), U_OK);
+  ck_assert_int_eq(ulfius_start_framework(&instance), U_OK);
+
+  sprintf(url, "ws://localhost:%d/%s", PORT_10, PREFIX_WEBSOCKET);
+  
+  // Test websocket connection without Origin header
+  ulfius_init_request(&request);
+  ulfius_init_response(&response);
+  ck_assert_int_eq(ulfius_set_websocket_request(&request, url, DEFAULT_PROTOCOL, DEFAULT_EXTENSION), U_OK);
+  ck_assert_int_eq(ulfius_open_websocket_client_connection(&request, NULL, NULL, &websocket_incoming_message_without_origin_callback_client, NULL, NULL, NULL, &websocket_client_handler, &response), U_OK);
+  ck_assert_int_eq(ulfius_websocket_client_connection_wait_close(&websocket_client_handler, 0), U_WEBSOCKET_STATUS_CLOSE);
+  ulfius_clean_request(&request);
+  ulfius_clean_response(&response);
+  
+  // Test websocket connection with Origin header
+  ulfius_init_request(&request);
+  ulfius_init_response(&response);
+  websocket_client_handler.websocket = NULL;
+  websocket_client_handler.response = NULL;
+  ck_assert_int_eq(ulfius_set_request_properties(&request, U_OPT_HEADER_PARAMETER, "Origin", ORIGIN, U_OPT_NONE), U_OK);
+  ck_assert_int_eq(ulfius_set_websocket_request(&request, url, DEFAULT_PROTOCOL, DEFAULT_EXTENSION), U_OK);
+  ck_assert_int_eq(ulfius_open_websocket_client_connection(&request, NULL, NULL, &websocket_incoming_message_with_origin_callback_client, NULL, NULL, NULL, &websocket_client_handler, &response), U_OK);
+  ck_assert_int_eq(ulfius_websocket_client_connection_wait_close(&websocket_client_handler, 0), U_WEBSOCKET_STATUS_CLOSE);
+  ulfius_clean_request(&request);
+  ulfius_clean_response(&response);
+  
+  ck_assert_int_eq(ulfius_stop_framework(&instance), U_OK);
+  ulfius_clean_instance(&instance);
+}
+END_TEST
+
+START_TEST(test_ulfius_websocket_json_messages)
+{
+  struct _u_instance instance;
+  struct _u_request request;
+  struct _u_response response;
+  struct _websocket_client_handler websocket_client_handler = {NULL, NULL};
+  char url[64];
+
+  ck_assert_int_eq(ulfius_init_instance(&instance, PORT_11, NULL, NULL), U_OK);
+  ck_assert_int_eq(ulfius_add_endpoint_by_val(&instance, "GET", PREFIX_WEBSOCKET, "/valid", 0, &callback_websocket_json_valid_messages, NULL), U_OK);
+  ck_assert_int_eq(ulfius_add_endpoint_by_val(&instance, "GET", PREFIX_WEBSOCKET, "/invalid", 0, &callback_websocket_json_invalid_messages, NULL), U_OK);
+  ck_assert_int_eq(ulfius_add_endpoint_by_val(&instance, "GET", PREFIX_WEBSOCKET, "/empty", 0, &callback_websocket_json_empty_messages, NULL), U_OK);
+  ck_assert_int_eq(ulfius_start_framework(&instance), U_OK);
+
+  // Valid JSON
+  sprintf(url, "ws://localhost:%d/%s/valid", PORT_11, PREFIX_WEBSOCKET);
+  ulfius_init_request(&request);
+  ulfius_init_response(&response);
+  ck_assert_int_eq(ulfius_set_websocket_request(&request, url, NULL, NULL), U_OK);
+  ck_assert_int_eq(ulfius_open_websocket_client_connection(&request, &websocket_manager_json_valid_callback, NULL, NULL, NULL, NULL, NULL, &websocket_client_handler, &response), U_OK);
+  ck_assert_int_eq(ulfius_websocket_client_connection_wait_close(&websocket_client_handler, 0), U_WEBSOCKET_STATUS_CLOSE);
+  ulfius_clean_request(&request);
+  ulfius_clean_response(&response);
+
+  usleep(50);
+
+  // Invalid JSON
+  sprintf(url, "ws://localhost:%d/%s/invalid", PORT_11, PREFIX_WEBSOCKET);
+  ulfius_init_request(&request);
+  ulfius_init_response(&response);
+  ck_assert_int_eq(ulfius_set_websocket_request(&request, url, NULL, NULL), U_OK);
+  ck_assert_int_eq(ulfius_open_websocket_client_connection(&request, &websocket_manager_json_invalid_callback, NULL, NULL, NULL, NULL, NULL, &websocket_client_handler, &response), U_OK);
+  ck_assert_int_eq(ulfius_websocket_client_connection_wait_close(&websocket_client_handler, 0), U_WEBSOCKET_STATUS_CLOSE);
+  ulfius_clean_request(&request);
+  ulfius_clean_response(&response);
+
+  usleep(50);
+
+  // Empty JSON
+  sprintf(url, "ws://localhost:%d/%s/empty", PORT_11, PREFIX_WEBSOCKET);
+  ulfius_init_request(&request);
+  ulfius_init_response(&response);
+  ck_assert_int_eq(ulfius_set_websocket_request(&request, url, NULL, NULL), U_OK);
+  ck_assert_int_eq(ulfius_open_websocket_client_connection(&request, &websocket_manager_json_empty_callback, NULL, NULL, NULL, NULL, NULL, &websocket_client_handler, &response), U_OK);
+  ck_assert_int_eq(ulfius_websocket_client_connection_wait_close(&websocket_client_handler, 0), U_WEBSOCKET_STATUS_CLOSE);
+  ulfius_clean_request(&request);
+  ulfius_clean_response(&response);
+
+  usleep(50);
+  ck_assert_int_eq(ulfius_stop_framework(&instance), U_OK);
+
+  ulfius_clean_instance(&instance);
+}
+END_TEST
 
 #endif
 
@@ -944,7 +1171,11 @@ static Suite *ulfius_suite(void)
 	tcase_add_test(tc_websocket, test_ulfius_websocket_extension_deflate);
 	tcase_add_test(tc_websocket, test_ulfius_websocket_extension_deflate_with_all_params);
 	tcase_add_test(tc_websocket, test_ulfius_websocket_extension_deflate_error_params);
+#ifndef U_DISABLE_WS_MESSAGE_LIST
 	tcase_add_test(tc_websocket, test_ulfius_websocket_keep_messages);
+#endif
+	tcase_add_test(tc_websocket, test_ulfius_websocket_origin);
+	tcase_add_test(tc_websocket, test_ulfius_websocket_json_messages);
 #endif
 	tcase_set_timeout(tc_websocket, 30);
 	suite_add_tcase(s, tc_websocket);
@@ -962,11 +1193,17 @@ int main(int argc, char *argv[])
   s = ulfius_suite();
   sr = srunner_create(s);
 
+  pthread_mutex_init(&ws_lock, NULL);
+  pthread_cond_init(&ws_cond, NULL);
+
   srunner_run_all(sr, CK_VERBOSE);
   number_failed = srunner_ntests_failed(sr);
   srunner_free(sr);
-  
   ulfius_global_close();
+  
+  pthread_mutex_destroy(&ws_lock);
+  pthread_cond_destroy(&ws_cond);
+
   //y_close_logs();
 	return (number_failed == 0) ? EXIT_SUCCESS : EXIT_FAILURE;
 }
